"use strict";

var SocketManager = {
    TAG: "SocketManager",

    socketIsConnect: false, // 手机Wifi是否连接socket
    socket: undefined, // socket

    recvDataCallback: null,
    recvInvalidDataCallback: null,
    connectCallbackArr: [], // 连接成功回调数组
    disconnectCallbackArr: [], // 断开连接回调数组

    /**
     * 是否socket已连接
     * @return 
     *  true:socket已连接 
     *  false:socket未连接
     */
    isSocketConnected: function isSocketConnected() {
        return this.socketIsConnect;
    },

    /**
     * 连接socket
     * @ip tcp 地址
     * @port tcp 端口
     * @succCallback 连接socket成功回调
     * @failCallback 连接socket失败回调
     */
    connectSocket: function connectSocket(ip, port, succCallback, failCallback) {
        var that = this;
        console.log(this.TAG + ": connectSocket " + ip + ", " + port);
        if (that.socketIsConnect == true) {
            console.log(that.TAG + ": already connect");
            succCallback();
        } else {
            console.log(that.TAG + ": connect");
            that.socket = new Socket();
            // 监听错误
            that.socket.onError = function (errorMsg) {
                socket.close();
                console.log(that.TAG + ": socket error, close: " + errorMsg);
                that.socketIsConnect = false;
                that.__disconnectCall();
            };
            // 监听关闭
            that.socket.onClose = function (hasError) {
                console.log(that.TAG + ": socket closed: " + hasError);
                that.socketIsConnect = false;
                that.__disconnectCall();
            };
            // sokect open
            that.socket.open(ip, 6000, function () {
                that.socketIsConnect = true;
                console.log(that.TAG + ": connectcallback " + that.socketIsConnect);
                that.__connectSuccCall();
                succCallback();
            }, function () {
                console.log(that.TAG + ": socket disconnected");
                that.socketIsConnect = false;
                that.__disconnectCall();
                failCallback();
            });
        }
    },

    /**
     * 断开socket
     * @succCallback 断开socket成功回调
     * @failCallback 断开socket失败回调
     */
    disconnectSocket: function disconnectSocket(succCallback, failCallback) {
        succCallback = succCallback || function () {};
        failCallback = failCallback || function () {};
        var that = this;
        console.log(this.TAG + ": disConnectSocket");
        if (that.socketIsConnect == false) {
            console.log(that.TAG + ": already disconnect");
            succCallback();
        } else {
            that.socketIsConnect = false;
            that.socket.close(succCallback, failCallback);
        }

        that.__disconnectCall();
    },

    /**
     * 发送数据
     * @data 发送的数据 byte数组  [0x11, 0x22, 0x33]
     *      该数据会校验，如果数据格式错误，会发送失败
     * @succCallback 发送成功的回调
     * @failCallback 发送失败的回调
     */
    sendData: function sendData(data, succCallback, failCallback) {
        succCallback = succCallback || function () {};
        failCallback = failCallback || function () {};
        // console.log("state " + this.socket.state)
        if (data[5] != 0x01 && data[5] != 0xA0) {
            // 不打印心跳和电平
            console.log(this.TAG + " => send data req: " + util.bytesToString(data));
        }
        if (this.socketIsConnect == true) {
            if (this.__checkValidData(data)) {
                this.socket.write(data, succCallback, failCallback);
            } else {
                failCallback("data check invalid: " + util.bytesToString(data));
            }
        } else {
            failCallback("socket not connect");
        }
    },

    /**
     * 注册接收数据函数
     * 收到完整的合法数据，会回调该函数
     * @callback 接收数据的回调函数  
     *      function(data) {
     *          // data为byte数组  [0x44, 0x55, 0x66, ... ]
     *      }
     */
    subscribeRecvData: function subscribeRecvData(callback, invalidCallback) {
        this.recvDataCallback = callback;
        this.recvInvalidDataCallback = invalidCallback;
    },

    /**
     * 取消接收数据监听函数
     * @callback 要取消的callback监听
     */
    unSubscribeRecvData: function unSubscribeRecvData() {
        this.recvDataCallback = null;
        this.recvInvalidDataCallback = null;
    },

    /**
     * 添加socket连接成功监听
     * @callback 成功连接回调
     */
    addSocketConnectEvent: function addSocketConnectEvent(callback) {
        if (typeof callback != 'function') {
            console.log(this.TAG + ": connectArr need function paramter");
        } else if (this.connectCallbackArr.indexOf(callback) > -1) {
            console.log(this.TAG + ": connectArr already registed");
        } else {
            this.connectCallbackArr.push(callback);
        }
    },

    /**
     * 取消socket连接成功监听
     * @callback 要取消的callback监听
     */
    removeSocketConnectEvent: function removeSocketConnectEvent(callback) {
        var index = this.connectCallbackArr.indexOf(callback);
        if (index > -1) {
            this.connectCallbackArr.splice(index, 1);
        }
    },

    /**
     * 添加socket断开连接监听
     * @callback 断开连接回调
     */
    addSocketDisconnectEvent: function addSocketDisconnectEvent(callback) {
        if (typeof callback != 'function') {
            console.log(this.TAG + ": disconnectArr need function paramter");
        } else if (this.disconnectCallbackArr.indexOf(callback) > -1) {
            console.log(this.TAG + ": disconnectArr already registed");
        } else {
            this.disconnectCallbackArr.push(callback);
        }
    },

    /**
     * 取消socket断开连接监听
     * @callback 要取消的callback监听
     */
    removeSocketDisconnectEvent: function removeSocketDisconnectEvent(callback) {
        var index = this.disconnectCallbackArr.indexOf(callback);
        if (index > -1) {
            this.disconnectCallbackArr.splice(index, 1);
        }
    },

    // *********************************************************************
    // ******** 以下为私有函数 **********************************************
    // *********************************************************************
    /**
     * 连接成功，执行函数
     */
    __buffer: [], // 数据接收buffer
    __connectSuccCall: function __connectSuccCall() {
        var that = this;
        that.__buffer = [];

        // 接收数据
        that.socket.onData = function (data) {
            for (var xx = 0; xx < data.length; xx++) {
                that.__buffer.push(data[xx]);
            }
            var singleData = null;
            while ((singleData = that.__getSingleDataFromBuffer()) != null) {
                if (that.__checkValidData(singleData)) {
                    if (data[5] != 0x01 && data[5] != 0xA0) {
                        console.log(that.TAG + " => recv data msg: " + util.bytesToString(singleData));
                    }
                    that.recvDataCallback(singleData);
                } else {
                    console.log(that.TAG + " => recv invalid data: " + util.bytesToString(singleData));
                    that.recvInvalidDataCallback(singleData);
                }
            }
        };

        // 回调socket连接成功的监听
        for (var i = 0; i < that.connectCallbackArr.length; i++) {
            var element = that.connectCallbackArr[i];
            if (typeof element == 'function') {
                try {
                    element();
                } catch (e) {
                    console.log(that.TAG + ": connectCallbackArr error " + e);
                }
            }
        }
    },

    /**
     * 断开连接执行函数
     */
    __disconnectCall: function __disconnectCall() {
        var that = this;
        that.__buffer = [];
        // 取消数据监听
        that.socket.onData = function () {};
        // 回调socket连接断开监听
        for (var i = 0; i < that.disconnectCallbackArr.length; i++) {
            var element = that.disconnectCallbackArr[i];
            if (typeof element == 'function') {
                try {
                    element();
                } catch (e) {
                    console.log(that.TAG + ": disconnectCallbackArr error " + e);
                }
            }
        }
    },

    /**
     * 从__buffer获取合法的命令数据
     */
    __HEAD_LEN: 7,
    __DATA_LEN_INDEX: 6,
    __TAIL_LEN: 4,
    __getSingleDataFromBuffer: function __getSingleDataFromBuffer() {
        var that = this;
        // 删除非法头部
        while (that.__buffer.length >= 3 && !(that.__buffer[0] == 0xf4 && that.__buffer[1] == 0xf5 && that.__buffer[2] == 0xf6)) {
            that.__buffer.splice(0);
        }
        // 解析一条数据
        if (that.__buffer.length >= that.__HEAD_LEN + that.__TAIL_LEN) {
            var dataLen = that.__buffer[that.__DATA_LEN_INDEX];
            var totalLen = that.__HEAD_LEN + that.__TAIL_LEN + dataLen;
            if (that.__buffer.length >= totalLen) {
                var validData = that.__buffer.splice(0, totalLen);
                return validData;
            }
        }
        return null;
    },

    /**
     * 检查命令是否合法
     * 数据校验：=（地址+帧命令+数据个数+数据1+数据2+…+数据n）&0x7F
     */
    __checkValidData: function __checkValidData(singleData) {
        var that = this;
        if (singleData.length < that.__HEAD_LEN) return false;
        var totalLen = that.__HEAD_LEN + singleData[that.__DATA_LEN_INDEX] + that.__TAIL_LEN;
        if (singleData.length < totalLen) return false;

        var unint8 = 0;
        for (var i = 4; i < singleData.length - that.__TAIL_LEN; i++) {
            var e = singleData[i];
            unint8 = unint8 + e;
        }
        var unit8Arr = new Uint8Array([unint8]);
        var check = unit8Arr[0] & 0x7f;
        if (check == singleData[totalLen - that.__TAIL_LEN]) {
            return true;
        } else {
            return false;
        }
    },

    /**
     * 获取校验位
     * 数据校验：=（地址+帧命令+数据个数+数据1+数据2+…+数据n）&0x7F
     * @singleData 发送的帧头+数据长度+数据
     * @return null-数据不合法 | check 校验值
     */
    getCheckVal: function getCheckVal(singleData) {
        var that = this;
        if (singleData.length < that.__HEAD_LEN) return null;
        var totalLen = that.__HEAD_LEN + singleData[that.__DATA_LEN_INDEX];
        if (singleData.length < totalLen) return null;

        var unint8 = 0;
        for (var i = 4; i < singleData.length; i++) {
            var e = singleData[i];
            unint8 = unint8 + e;
        }
        var unit8Arr = new Uint8Array([unint8]);
        var check = unit8Arr[0] & 0x7f;
        return check;
    }

};