'use strict';

var TcpManger = {
    /**
     * 搜索设备
     */
    __searchFinishCallback: undefined,
    searchDevice: function searchDevice(finishCallback, failCallback) {
        finishCallback = finishCallback || function () {};
        failCallback = failCallback || function () {};
        var that = this;
        that.__searchFinishCallback = finishCallback;
        // 获取wifi地址（移动端设备，通过WIFI加入机器所在网络）
        networkinterface.getWiFiIPAddress(function (ipinterface) {
            console.log('succ:' + ipinterface.ip);
            that.__searchDevice(ipinterface.ip);
        }, function (error) {
            console.log('error:' + error);
            navigator.notification.alert('好像没有连接wifi哦，请检查后重试', // message
            null, '提示');
            failCallback();
        });
    },

    /**
     * 根据ip地址，搜索设备
     */
    __searchDevice: function __searchDevice(ip) {
        var that = this;
        var lastDotIndex = ip.lastIndexOf('.');
        var prefix = ip.slice(0, lastDotIndex);
        that.__reset();
        for (var i = 1; i < 255; i++) {
            var tryIp = prefix + '.' + i;
            that.__connectSocket(tryIp);
        }
    },

    // 设备列表
    __hasFinishNum: 0,
    __deviceList: [],
    __deviceIPList: [],
    __token: 0,
    __updateFinishNum: function __updateFinishNum() {
        var that = this;
        this.__hasFinishNum++;
        // console.log('cur num ' + this.__hasFinishNum)
        if (this.__hasFinishNum == 254) {
            console.log('搜索结束 deviceListLen: ' + this.__deviceList.length);
            var deviceArr = [];
            for (var i = 0; i < this.__deviceList.length; i++) {
                var deviceData = this.__deviceList[i];
                var deviceItem = {};
                if (deviceData[7] != DeviceManager.getDeviceCode()) continue;
                deviceItem.id = deviceData[8];
                var nameData = deviceData;
                deviceItem.name = String.fromCharCode.apply(String, nameData.subarray(9, 28));
                deviceItem.version = deviceData[30] + '.' + deviceData[31] + '.' + deviceData[32];
                deviceItem.ip = that.__deviceIPList[i];
                deviceItem.state = deviceData[29];
                deviceArr.push(deviceItem);
            }
            DeviceManager.setDeviceList(deviceArr);
            if (undefined != this.__searchFinishCallback) {
                this.__searchFinishCallback();
            }
        }
    },

    // 重置搜索设备
    __reset: function __reset() {
        var that = this;
        that.__hasFinishNum = 0;
        that.__deviceList = [];
        that.__deviceIPList = [];
        that.__token = new Date().getTime();
    },

    /**
     * 尝试连接socket
     */
    __connectSocket: function __connectSocket(ip) {
        var that = this;
        var tmpSocket = new Socket();
        var tmpSocketValid = true;
        var finalToken = that.__token;
        var failFinish = function failFinish() {
            if (finalToken != that.__token) return;
            if (!tmpSocketValid) return;
            tmpSocketValid = false;
            that.__updateFinishNum();
        };
        var succFinish = function succFinish(data) {
            if (finalToken != that.__token) return;
            if (!tmpSocketValid) return;
            tmpSocketValid = false;
            that.__updateFinishNum();
            that.__deviceList.push(data);
            that.__deviceIPList.push(ip);
        };
        tmpSocket.onData = function (data) {
            //invoked after new batch of data is received (typed array of bytes Uint8Array)
            console.log("recv data " + ip);
            console.log('recvData: ' + util.bytesToString(data));
            if (data[5] == 0x01) {
                succFinish(data);
                tmpSocket.close(); // 关闭socket
            }
        };
        tmpSocket.onError = function (errorMessage) {
            //invoked after error occurs during connection
            console.log("onError " + ip + errorMessage);
            failFinish();
        };
        tmpSocket.onClose = function (hasError) {
            //invoked after connection close
            console.log("onClose " + ip + hasError);
            failFinish();
        };
        tmpSocket.open(ip, 6000, function () {
            console.log("connect succ " + ip);
            var sendData = dmx108_protocol.getSearchDevicePro();
            console.log('sendData: ' + util.bytesToString(sendData));
            tmpSocket.write(sendData, function () {
                console.log('sendSucc' + ip);
            }, function (error) {
                console.log('sendFail: ' + ip + error);
            });
        }, function (error) {
            // console.log("connect fail " + ip + error)
            failFinish();
        });
        setTimeout(function () {
            if (tmpSocket.state == Socket.State.OPENING) {
                failFinish();
            } else if (tmpSocket.state == Socket.State.OPENED) {
                failFinish();
                tmpSocket.close();
            }
        }, 1500);
    }
};