'use strict';

var util = {
    /**
     * byte数组 转 16进制字符串
     * 输入：[0x11, 0x22, 0x33]
     * 输出：11 22 33 
     */
    bytesToString: function bytesToString(bytes) {
        return Array.prototype.map.call(bytes, function (x) {
            return ('0' + x.toString(16)).slice(-2) + ' ';
        }).join('').toUpperCase();
    },

    /**
     * 字符串转String，非数组
     * @param {char数组} bytes 
     */
    getStringByArr: function getStringByArr(bytes) {
        return String.fromCharCode.apply(String, bytes);
    },

    /**
     * 获取中英文混合的字符串
     * @param {需要转换的字符数组} bytes 
     */
    getMixStringByBytes: function getMixStringByBytes(bytes) {
        try {
            var newBytes = [];
            for (var i = 0; i < bytes.length; i++) {
                if (bytes[i] == 0) continue;
                newBytes.push(bytes[i]);
            }
            var str = '%' + this.bytesToString(newBytes).trim().replace(RegExp(' ', 'g'), '%');
            return $URL.decode(str);
        } catch (e) {
            return '';
        }
    },

    /**
     * 将中英文字符串转换成byte数组
     * @param {中英文混合的字符串} mixStr 
     */
    getBytesByMixString: function getBytesByMixString(mixStr) {
        var str = $URL.encode(mixStr);
        var res = [];
        for (var i = 0; i < str.length;) {
            if (str[i] == '%') {
                var gb1 = str[i + 1];
                var gb2 = str[i + 2];
                res.push(parseInt('0x' + gb1 + gb2));
                i += 3;
            } else {
                res.push(str.charCodeAt(i));
                i++;
            }
        }
        return res;
    },

    /** 
     * 建议长度单位rem。竖屏：屏幕宽度==37.5rem；横屏：屏幕高度==37.5rem。
     * rem也即一倍字体大小，其值不能太小（建议9px以上），否则浏览器不接受，故而使用37.5等分。
     */
    setHtmlFontSize: function setHtmlFontSize(window, scale) {
        // console.log(window);

        console.log(window.screen);
        var refPx = Math.min(window.screen.availHeight, window.screen.availWidth); //横屏
        // if (window.screen.orientation.angle === 180 || window.screen.orientation.angle === 0) {
        //     refPx = window.screen.availWidth; //竖屏
        // }
        var DEFAULT_FONT_SIZE = 16;
        var remPx = refPx / 37.5 * (scale || 1); //(1)rem == (remPx)px
        window.document.documentElement.style.fontSize = remPx + "px";
        console.log(" ====== html fontSize (rem) = " + window.document.documentElement.style.fontSize);
    },

    //return { name:value, ... } 注意：value都是字符串，解析数字参数需要调用转换函数，比如parseInt
    parseQueryString: function parseQueryString(url) {
        //获取url中"?"字符后的参数
        var ret = {};
        var index = url.indexOf("?");
        if (index >= 0) {
            var paramStr = url.substring(index + 1);
            var paramArr = paramStr.split("&");
            for (var i = 0; i < paramArr.length; ++i) {
                var param = paramArr[i];
                var paramPair = param.split("=");
                var paramName = paramPair[0];
                var paramValue = paramPair[1];
                ret[paramName] = paramValue;
            }
        }
        return ret;
    }

};