'use strict';

//getter和setter函数通常是给界面调用，notify函数则是给数据层调用
var dmx108_dataApi = {

    /**
     * 获取设备列表
     * [{id:0, name:'', version:'', ip:'', state:0}]
     */
    getDeviceList: function getDeviceList() {
        return DeviceManager.getDeviceList();
    },


    /**
     * 获取场景列表
     * [{id:0, isPreset:ture, name:''}]
     */
    getSceneList: function getSceneList() {
        var programArr = [];
        var programList = dmx108_data.getProgramDataList();
        for (var i = 1; i < programList.length; i++) {
            var programData = programList[i];
            var programItem = {};
            programItem.id = programData[7];
            programItem.isPreset = programData[8];
            if (programData[8] == 1) {
                programItem.name = util.getMixStringByBytes(programData.slice(9, 29));
            } else {
                programItem.name = '';
            }
            programArr.push(programItem);
        }
        return programArr;
    },


    // 获取当前场景信息
    getCurScene: function getCurScene() {
        return dmx108_data.getCurSceneInfo();
    },


    //获取输入/输出通道列表 [{list:[], volumeIndex:0, isMute:false, channelName:'', electricPercent:0}]
    getVolumeInfoList: function getVolumeInfoList() {
        var dataList = dmx108_data.getChannelList();
        var list = [];
        for (var i = 0; i < dataList.length; i++) {
            var data = dataList[i];
            var item = {};
            item.list = dmx108_paramters.GAIN_ENUM;
            item.isMute = (data[10] & 0x80) != 0;
            item.volumeIndex = ((data[10] & 0x7F) << 8) + data[9];
            item.channelName = util.getMixStringByBytes(data.slice(36, 46));
            item.electricPercent = 0;
            list.push(item);
        }
        return list;
    },


    //设置输入/输出通道音量。输入通道索引在前，输出通道索引在后
    setVolumeIndex: function setVolumeIndex(channelIndex, volumnIndex) {
        channelIndex = parseInt(channelIndex);
        volumnIndex = parseInt(volumnIndex);
        // 更新本地缓存
        dmx108_data.__channelInfoList[channelIndex][9] = volumnIndex & 0x00FF;
        var isMute = (dmx108_data.__channelInfoList[channelIndex][10] & 0x80) != 0;
        dmx108_data.__channelInfoList[channelIndex][10] = (volumnIndex & 0xFF00) >> 8;
        if (isMute) {
            dmx108_data.__channelInfoList[channelIndex][10] = dmx108_data.__channelInfoList[channelIndex][10] | 0x80;
        }
        // 设置机器数据
        var divid = DeviceManager.getDeviceCode() == dmx108_protocol.DEVICE_CODE_ENUM.DMX108 ? 8 : 16;
        if (channelIndex >= divid) {
            channelIndex = channelIndex - divid + 0x10;
        }
        SocketManager.sendData(dmx108_protocol.getSetChannelGainPro(channelIndex, isMute ? 1 : 0, volumnIndex));
    },


    //设置输入/输出通道静音。输入通道索引在前，输出通道索引在后
    setMute: function setMute(channelIndex, isMute) {
        // 更新本地缓存
        if (isMute) {
            dmx108_data.__channelInfoList[channelIndex][10] = dmx108_data.__channelInfoList[channelIndex][10] | 0x80;
        } else {
            dmx108_data.__channelInfoList[channelIndex][10] = dmx108_data.__channelInfoList[channelIndex][10] & 0x7F;
        }
        // 设置机器数据
        var data = dmx108_data.__channelInfoList[channelIndex];
        var volumnIndex = ((data[10] & 0x7F) << 8) + data[9];
        var divid = DeviceManager.getDeviceCode() == dmx108_protocol.DEVICE_CODE_ENUM.DMX108 ? 8 : 16;
        if (channelIndex >= divid) {
            channelIndex = channelIndex - divid + 0x10;
        }
        SocketManager.sendData(dmx108_protocol.getSetChannelGainPro(channelIndex, isMute ? 1 : 0, volumnIndex));
    },


    //由界面注册此回调函数
    setElectricHandler: function setElectricHandler(handler) {
        this.electricHandler = handler;
    },


    //由数据层定时器执行此通知函数
    notifyElectric: function notifyElectric(channelIndex, electricPercent) {
        if (this.electricHandler) this.electricHandler(channelIndex, electricPercent);
    },


    //获取混音开关列表信息
    // $scope.mix_inNameList_8CH = ["IN1","IN2","IN3","IN4","IN5","IN6","IN7","IN8","AutoMixer","Ducker",];
    // $scope.mix_inNameList_16CH = ["IN1","IN2","IN3","IN4","IN5","IN6","IN7","IN8","IN9","IN10","IN11","IN12","IN13","IN14","IN15","IN16","AutoMixer","Ducker","AFC1","AFC2","AFC3","AFC4","AEC",];
    getMixOutList_2d: function getMixOutList_2d() {
        var outputList = [];
        var is8ch = DeviceManager.getDeviceCode() == dmx108_protocol.DEVICE_CODE_ENUM.DMX108;
        if (is8ch) {
            for (var i = 0; i < 8; ++i) {
                var inputList = [];
                var data = dmx108_data.__channelInfoList[8 + i];
                for (var j = 0; j < 8; ++j) {
                    inputList.push((data[49 + 2 * j] & 0x80) != 0);
                }
                inputList.push((data[47] & 0x80) != 0);
                inputList.push((data[29] & 0x80) != 0);
                outputList.push(inputList);
            }
        } else {
            for (var _i = 0; _i < 16; ++_i) {
                var _inputList = [];
                var _data = dmx108_data.__channelInfoList[16 + _i];
                var in1_8 = _data[72];
                var in9_16 = _data[73];
                var autoMixAndDocker = _data[74];
                var afc1_8 = _data[75];
                var aec_1_8 = _data[76];
                for (var _i2 = 0; _i2 < 8; _i2++) {
                    _inputList.push((in1_8 & 0x01 << _i2) != 0);
                }
                for (var _i3 = 0; _i3 < 8; _i3++) {
                    _inputList.push((in9_16 & 0x01 << _i3) != 0);
                }
                _inputList.push((autoMixAndDocker & 0x01) != 0); // AutoMixer
                _inputList.push((autoMixAndDocker & 0x02) != 0); // Ducker
                for (var _i4 = 0; _i4 < 4; _i4++) {
                    // AFC 1-4
                    _inputList.push((afc1_8 & 0x01 << _i4) != 0);
                }
                for (var _i5 = 0; _i5 < 1; _i5++) {
                    // AEC 1
                    _inputList.push((aec_1_8 & 0x01 << _i5) != 0);
                }
                outputList.push(_inputList);
            }
        }
        return outputList;
    },


    //打开或关闭某项混音
    setMixItem: function setMixItem(inChIndex, outChIndex, isEnable) {
        var is8ch = DeviceManager.getDeviceCode() == dmx108_protocol.DEVICE_CODE_ENUM.DMX108;
        if (is8ch) {
            var data = dmx108_data.__channelInfoList[8 + outChIndex];
            if (inChIndex < 8) {
                var volumnIndex = ((data[49 + inChIndex * 2] & 0x7F) << 8) + data[48 + inChIndex * 2];
                if (isEnable) {
                    dmx108_data.__channelInfoList[8 + outChIndex][49 + inChIndex * 2] = dmx108_data.__channelInfoList[8 + outChIndex][49 + inChIndex * 2] | 0x80;
                } else {
                    dmx108_data.__channelInfoList[8 + outChIndex][49 + inChIndex * 2] = dmx108_data.__channelInfoList[8 + outChIndex][49 + inChIndex * 2] & 0x7F;
                }
                SocketManager.sendData(dmx108_protocol.getSetMixPro(0x10 + outChIndex, inChIndex, isEnable ? 1 : 0, volumnIndex));
            } else if (inChIndex == 8) {
                var _volumnIndex = ((data[47] & 0x7F) << 8) + data[46];
                if (isEnable) {
                    dmx108_data.__channelInfoList[8 + outChIndex][47] = dmx108_data.__channelInfoList[8 + outChIndex][41] | 0x80;
                } else {
                    dmx108_data.__channelInfoList[8 + outChIndex][47] = dmx108_data.__channelInfoList[8 + outChIndex][41] & 0x7F;
                }
                SocketManager.sendData(dmx108_protocol.getSetMixPro(0x10 + outChIndex, 0x20, isEnable ? 1 : 0, _volumnIndex));
            } else if (inChIndex == 9) {
                var _volumnIndex2 = ((data[29] & 0x7F) << 8) + data[28];
                if (isEnable) {
                    dmx108_data.__channelInfoList[8 + outChIndex][29] = dmx108_data.__channelInfoList[8 + outChIndex][29] | 0x80;
                } else {
                    dmx108_data.__channelInfoList[8 + outChIndex][29] = dmx108_data.__channelInfoList[8 + outChIndex][29] & 0x7F;
                }
                SocketManager.sendData(dmx108_protocol.getSetMixPro(0x10 + outChIndex, 0x21, isEnable ? 1 : 0, _volumnIndex2));
            }
        } else {
            var _data2 = dmx108_data.__channelInfoList[16 + outChIndex];
            if (inChIndex < 4) {
                var _volumnIndex3 = ((_data2[23 + inChIndex * 2] & 0x7F) << 8) + _data2[22 + inChIndex * 2];
                if (isEnable) {
                    dmx108_data.__channelInfoList[16 + outChIndex][72] = dmx108_data.__channelInfoList[16 + outChIndex][72] | 0x01 << inChIndex;
                } else {
                    dmx108_data.__channelInfoList[16 + outChIndex][72] = dmx108_data.__channelInfoList[16 + outChIndex][72] & (0xff ^ 0x01 << inChIndex);
                }
                SocketManager.sendData(dmx108_protocol.getSetMixPro(0x10 + outChIndex, inChIndex, isEnable ? 1 : 0, _volumnIndex3));
            } else if (inChIndex < 8) {
                var _volumnIndex4 = ((_data2[47 + (inChIndex - 4) * 2] & 0x7F) << 8) + _data2[47 + (inChIndex - 4) * 2];
                if (isEnable) {
                    dmx108_data.__channelInfoList[16 + outChIndex][72] = dmx108_data.__channelInfoList[16 + outChIndex][72] | 0x01 << inChIndex;
                } else {
                    dmx108_data.__channelInfoList[16 + outChIndex][72] = dmx108_data.__channelInfoList[16 + outChIndex][72] & (0xff ^ 0x01 << inChIndex);
                }
                SocketManager.sendData(dmx108_protocol.getSetMixPro(0x10 + outChIndex, inChIndex, isEnable ? 1 : 0, _volumnIndex4));
            } else if (inChIndex < 16) {
                var _volumnIndex5 = ((_data2[47 + (inChIndex - 4) * 2] & 0x7F) << 8) + _data2[46 + (inChIndex - 4) * 2];
                if (isEnable) {
                    dmx108_data.__channelInfoList[16 + outChIndex][73] = dmx108_data.__channelInfoList[16 + outChIndex][73] | 0x01 << inChIndex - 8;
                } else {
                    dmx108_data.__channelInfoList[16 + outChIndex][73] = dmx108_data.__channelInfoList[16 + outChIndex][73] & (0xff ^ 0x01 << inChIndex - 8);
                }
                SocketManager.sendData(dmx108_protocol.getSetMixPro(0x10 + outChIndex, inChIndex, isEnable ? 1 : 0, _volumnIndex5));
            } else if (inChIndex == 16) {
                var _volumnIndex6 = ((_data2[71] & 0x7F) << 8) + _data2[70];
                if (isEnable) {
                    dmx108_data.__channelInfoList[16 + outChIndex][74] = dmx108_data.__channelInfoList[16 + outChIndex][74] | 0x01;
                } else {
                    dmx108_data.__channelInfoList[16 + outChIndex][74] = dmx108_data.__channelInfoList[16 + outChIndex][74] & 0x02;
                }
                SocketManager.sendData(dmx108_protocol.getSetMixPro(0x10 + outChIndex, 0x20, isEnable ? 1 : 0, _volumnIndex6));
            } else if (inChIndex == 17) {
                var _volumnIndex7 = ((_data2[78] & 0x7F) << 8) + _data2[77];
                if (isEnable) {
                    dmx108_data.__channelInfoList[16 + outChIndex][74] = dmx108_data.__channelInfoList[16 + outChIndex][74] | 0x02;
                } else {
                    dmx108_data.__channelInfoList[16 + outChIndex][74] = dmx108_data.__channelInfoList[16 + outChIndex][74] & 0x01;
                }
                SocketManager.sendData(dmx108_protocol.getSetMixPro(0x10 + outChIndex, 0x21, isEnable ? 1 : 0, _volumnIndex7));
            } else if (inChIndex == 18) {
                var _volumnIndex8 = ((_data2[80] & 0x7F) << 8) + _data2[79];
                if (isEnable) {
                    dmx108_data.__channelInfoList[16 + outChIndex][75] = dmx108_data.__channelInfoList[16 + outChIndex][75] | 0x01;
                } else {
                    dmx108_data.__channelInfoList[16 + outChIndex][75] = dmx108_data.__channelInfoList[16 + outChIndex][75] & 0xfe;
                }
                SocketManager.sendData(dmx108_protocol.getSetMixPro(0x10 + outChIndex, 0x30, isEnable ? 1 : 0, _volumnIndex8));
            } else if (inChIndex == 19) {
                var _volumnIndex9 = ((_data2[82] & 0x7F) << 8) + _data2[81];
                if (isEnable) {
                    dmx108_data.__channelInfoList[16 + outChIndex][75] = dmx108_data.__channelInfoList[16 + outChIndex][75] | 0x02;
                } else {
                    dmx108_data.__channelInfoList[16 + outChIndex][75] = dmx108_data.__channelInfoList[16 + outChIndex][75] & 0xfd;
                }
                SocketManager.sendData(dmx108_protocol.getSetMixPro(0x10 + outChIndex, 0x31, isEnable ? 1 : 0, _volumnIndex9));
            } else if (inChIndex == 20) {
                var _volumnIndex10 = ((_data2[84] & 0x7F) << 8) + _data2[83];
                if (isEnable) {
                    dmx108_data.__channelInfoList[16 + outChIndex][75] = dmx108_data.__channelInfoList[16 + outChIndex][75] | 0x04;
                } else {
                    dmx108_data.__channelInfoList[16 + outChIndex][75] = dmx108_data.__channelInfoList[16 + outChIndex][75] & 0xfb;
                }
                SocketManager.sendData(dmx108_protocol.getSetMixPro(0x10 + outChIndex, 0x32, isEnable ? 1 : 0, _volumnIndex10));
            } else if (inChIndex == 21) {
                var _volumnIndex11 = ((_data2[86] & 0x7F) << 8) + _data2[85];
                if (isEnable) {
                    dmx108_data.__channelInfoList[16 + outChIndex][75] = dmx108_data.__channelInfoList[16 + outChIndex][75] | 0x08;
                } else {
                    dmx108_data.__channelInfoList[16 + outChIndex][75] = dmx108_data.__channelInfoList[16 + outChIndex][75] & 0xf7;
                }
                SocketManager.sendData(dmx108_protocol.getSetMixPro(0x10 + outChIndex, 0x33, isEnable ? 1 : 0, _volumnIndex11));
            } else if (inChIndex == 22) {
                var _volumnIndex12 = ((_data2[88] & 0x7F) << 8) + _data2[87];
                if (isEnable) {
                    dmx108_data.__channelInfoList[16 + outChIndex][76] = dmx108_data.__channelInfoList[16 + outChIndex][76] | 0x01;
                } else {
                    dmx108_data.__channelInfoList[16 + outChIndex][76] = dmx108_data.__channelInfoList[16 + outChIndex][76] & 0xfe;
                }
                SocketManager.sendData(dmx108_protocol.getSetMixPro(0x10 + outChIndex, 0x40, isEnable ? 1 : 0, _volumnIndex12));
            }
        }
    },


    // 设置通道名
    setChannelName: function setChannelName(channelNo, channelName) {
        var tail = 10 - channelName.length;
        for (var i = 0; i < tail; i++) {
            channelName.push(0x00);
        }
        for (var _i6 = 0; _i6 < channelName.length; _i6++) {
            dmx108_data.__channelInfoList[channelNo][36 + _i6] = channelName[_i6];
        }
        var channelNum = DeviceManager.getDeviceCode() == dmx108_protocol.DEVICE_CODE_ENUM.DMX108 ? 8 : 16;
        if (channelNo < channelNum) {
            SocketManager.sendData(dmx108_protocol.getModigyChannelNamePro(channelNo, channelName));
        } else {
            SocketManager.sendData(dmx108_protocol.getModigyChannelNamePro(channelNo - channelNum + 0x10, channelName));
        }
    },


    //获取是否自动登录
    getAutoLogin: function getAutoLogin() {
        var curUserInfo = dmx108_data.getUserLoginInfo();
        var userInfoList = dmx108_data.getUserInfoList();
        var autoLogin = false;
        userInfoList.forEach(function (item) {
            if (curUserInfo != undefined && curUserInfo.id == item.id) {
                if (item.autoLogin) autoLogin = true;
            }
        });
        return autoLogin;
    },


    //设置自动登录
    setAutoLogin: function setAutoLogin(autoLogin) {
        SocketManager.sendData(dmx108_protocol.getUserAutoLoginPro(autoLogin ? 1 : 0));
    },
    login: function login(userId, password) {
        SocketManager.sendData(dmx108_protocol.getUserLoginPro(userId, password));
    },
    modifyPassword: function modifyPassword(password) {
        console.log(dmx108_data.getUserLoginInfo());
        console.log(dmx108_data.getUserInfoList());
        var curLoginUserInfo = dmx108_data.getUserLoginInfo();
        var userList = dmx108_data.getUserInfoList();
        if (curLoginUserInfo != undefined) {
            for (var i = 0; i < userList.length; i++) {
                if (curLoginUserInfo.id == userList[i].id) {
                    SocketManager.sendData(dmx108_protocol.getModifyPassword(util.getBytesByMixString(userList[i].userName), password));
                }
            }
        }
    }
};