"use strict";

/**
 * 协议相关模块
 */
var dmx108_protocol = {
    DEVICE_CODE_ENUM: {
        DMX108: 0x08,
        DMX116: 0x16
    },

    DEVICE_ADDRESS: 0x01,

    SEND_HEAD: [0xF1, 0xF2, 0xF3],
    SEND_TAIL: [0xF4, 0xF5, 0xF6],

    RECV_HEAD: [0xF4, 0xF5, 0xF6],
    RECV_TAIL: [0xF1, 0xF2, 0xF3],

    /**
     * 获取搜索设备的命令
     */
    getSearchDevicePro: function getSearchDevicePro() {
        var cmd = 0x01;
        var data = [];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取联机的命令
     */
    getConnectDevicePro: function getConnectDevicePro() {
        var cmd = 0x02;
        var data = [];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取取消联机的命令
     */
    getDisconnectDevicePro: function getDisconnectDevicePro() {
        var cmd = 0x03;
        var data = [];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取调用程式的命令
     * @{程式ID}programNo
     */
    getChangeProgramPro: function getChangeProgramPro(programNo) {
        programNo = parseInt(programNo);
        var cmd = 0x01;
        var data = [];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取修改设备名称的协议
     */
    getModigyDeviceNamePro: function getModigyDeviceNamePro(newDeviceName) {
        var cmd = 0x11;
        var data = newDeviceName;

        return this.__packageData(cmd, data);
    },

    /**
     * 获取修改通道名称的协议
     */
    getModigyChannelNamePro: function getModigyChannelNamePro(channelNo, newChannelName) {
        channelNo = parseInt(channelNo);
        var cmd = 0x12;
        var data = [channelNo];
        data = data.concat(newChannelName);

        return this.__packageData(cmd, data);
    },

    /**
     * 获取设置配置参数的协议(是否自动脱机)
     */
    getSetConfigPro: function getSetConfigPro(enableIndex) {
        enableIndex = parseInt(enableIndex);
        var cmd = 0x13;
        var data = [enableIndex];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取设置通道音量的协议
     */
    getSetChannelGainPro: function getSetChannelGainPro(channelNo, muteIndex, gainIndex) {
        channelNo = parseInt(channelNo);
        muteIndex = parseInt(muteIndex);
        gainIndex = parseInt(gainIndex);
        var cmd = 0x50;
        var data = [channelNo, muteIndex, gainIndex & 0x00FF, (gainIndex & 0xFF00) >> 8];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取设置混音器的协议
     */
    getSetMixPro: function getSetMixPro(outChannelNo, inChannelNo, muteIndex, gainIndex) {
        inChannelNo = parseInt(inChannelNo);
        outChannelNo = parseInt(outChannelNo);
        muteIndex = parseInt(muteIndex);
        gainIndex = parseInt(gainIndex);
        var cmd = 0x54;
        var data = [outChannelNo, inChannelNo, muteIndex, gainIndex & 0x00FF, (gainIndex & 0xFF00) >> 8];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取读取输入输出通道电平协议
     */
    getReadElectricPro: function getReadElectricPro() {
        var cmd = 0xA0;
        var data = [];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取读取程式数据协议
     */
    getReadProgramDataPro: function getReadProgramDataPro(programNo) {
        programNo = parseInt(programNo);
        var cmd = 0xE0;
        var data = [programNo];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取读取程式数据输入频道协议
     */
    getInChannelDataPro: function getInChannelDataPro(programNo, inputChannel) {
        programNo = parseInt(programNo);
        inputChannel = parseInt(inputChannel);
        var cmd = 0xE2;
        var data = [programNo, inputChannel];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取读取程式数据输出频道协议
     */
    getOutChannelDataPro: function getOutChannelDataPro(programNo, outputChannel) {
        programNo = parseInt(programNo);
        outputChannel = parseInt(outputChannel);
        var cmd = 0xE4;
        var data = [programNo, outputChannel];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取程式调用的协议
     */
    getLoadProgramPro: function getLoadProgramPro(programNo) {
        programNo = parseInt(programNo);
        var cmd = 0x62;
        var data = [programNo];

        return this.__packageData(cmd, data);
    },

    // ====================================================================================
    // == 用户相关 =============
    /**
     * 获取修改用户名密码的协议
     */
    getModifyPassword: function getModifyPassword(userName, password) {
        var cmd = 0x8A;
        var data = [0x02];
        var nameLen = userName.length;
        var nameTail = 16 - nameLen;
        for (var i = 0; i < nameTail; i++) {
            userName.push(0x00);
        }
        var passLen = password.length;
        var passTail = 16 - passLen;
        for (var _i = 0; _i < passTail; _i++) {
            password.push(0x00);
        }
        data.push(userName.length);
        data = data.concat(userName);
        data.push(passLen);
        data = data.concat(password);

        return this.__packageData(cmd, data);
    },

    /**
     * 获取查询数量的协议
     */
    getUserNumPro: function getUserNumPro() {
        var cmd = 0x8A;
        var data = [0x05];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取查询信息的协议
     */
    getUserInfoPro: function getUserInfoPro(userSeq) {
        userSeq = parseInt(userSeq);
        var cmd = 0x8A;
        var data = [0x06, userSeq];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取用户登陆的协议
     */
    getUserLoginPro: function getUserLoginPro(userId, password) {
        userId = parseInt(userId);
        var passLen = password.length;
        var tail = 16 - passLen;
        if (tail > 0) {
            for (var i = 0; i < tail; i++) {
                password.push(0x00);
            }
        }
        var cmd = 0x8A;
        var data = [0x07, userId, passLen];
        data = data.concat(password);

        return this.__packageData(cmd, data);
    },

    /**
     * 获取用户退出的协议
     */
    getUserLogoutPro: function getUserLogoutPro(userId) {
        userId = parseInt(userId);
        var cmd = 0x8A;
        var data = [0x08, userId];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取查询用户登陆状态的协议
     */
    getUserLoginInfoPro: function getUserLoginInfoPro() {
        var cmd = 0x8A;
        var data = [0x09];

        return this.__packageData(cmd, data);
    },

    /**
     * 获取设置是否自动登录的协议
     */
    getUserAutoLoginPro: function getUserAutoLoginPro(autoLoginIndex) {
        autoLoginIndex = parseInt(autoLoginIndex);
        var cmd = 0x8A;
        var data = [0x0B, autoLoginIndex];

        return this.__packageData(cmd, data);
    },

    // ===================================================================================
    /**
     * 获取心跳协议
     */
    getHeartbeatPro: function getHeartbeatPro() {
        var cmd = 0xA0;
        var data = [];

        return this.__packageData(cmd, data);
    },

    // =============================================================================================
    // =============================================================================================
    // =============================================================================================

    /**
     * 添加签名和帧尾
     */
    __packageData: function __packageData(cmd, data) {
        var sendData = [].concat(this.SEND_HEAD).concat([DeviceManager.getDeviceCode()]).concat(this.DEVICE_ADDRESS).concat([cmd]).concat([data.length]).concat(data);
        var check = SocketManager.getCheckVal(sendData);
        sendData = sendData.concat([check]).concat(this.SEND_TAIL);
        return sendData;
    }
};